from django.db import models
from django.contrib import databrowse
from django.db.models import Sum, Avg

class Manifest(models.Model):
    month = models.CharField(max_length=3)
    year = models.IntegerField(max_length=4)
    filename = models.CharField(max_length=256)

    def __unicode__(self):
        return self.filename

TYPE_CHOICES = (
    ('Import', 'Import'),
    ('Export', 'Export'),
    ('Rexport', 'Rexport'),
    )

#['type', 'number', 'date', 'ship_name', 'captain', 'flag', 'trader', 'port', 'country']
class Ship(models.Model):
    manifest_file = models.ForeignKey(Manifest)
    bill_type = models.CharField(max_length=50, choices=TYPE_CHOICES)
    number = models.IntegerField(max_length=20)
    date = models.DateField()
    ship_name = models.CharField(max_length=255)
    ship_name_trans = models.CharField(max_length=255, blank=True, null=True)
    captain = models.CharField(max_length=255)
    captain_trans = models.CharField(max_length=255, blank=True, null=True)
    flag = models.CharField(max_length=255)
    owner = models.CharField(max_length=255)
    owner_trans = models.CharField(max_length=255, blank=True, null=True)
    port = models.CharField(max_length=255)
    country = models.CharField(max_length=255)
    #calculated fields:
#    total_weight = models.IntegerField(null=True, blank=True, editable=False)
#    no_of_goods = models.IntegerField(null=True, blank=True, editable=False)
#    no_of_packages = models.IntegerField(null=True, blank=True, editable=False)

    def __unicode__(self):
        return "%d: %s" % (self.number, self.ship_name,)

    def get_ship_name(self):
        if self.ship_name_trans is not None:
            return self.ship_name_trans
        else:
            return self.ship_name
            

    def get_captain(self):
        if self.captain_trans is not None:
            return self.captain_trans
        else:
            return self.captain


    def get_owner(self):
        if self.owner_trans is not None:
            return self.owner_trans  
        else:
            return self.owner


    @property
    def total_weight(self):
        return self.good_set.all().aggregate(Sum('weight'))['weight__sum']


#    def calculate_weight(self):
#        self.total_weight = self.good_set.all().aggregate(Sum('weight'))
#        return self

#  class Meta:
#    ordering = ['number']

    def no_of_goods(self):
        return self.good_set.count()

#['description', 'package_type', 'no_of_packages', 'weight', 'value']
class Good(models.Model):
    ship = models.ForeignKey(Ship)
    description = models.TextField()
    description_string = models.TextField(blank=True, null=True)
    description_string_trans = models.TextField(blank=True, null=True)
    package_type = models.CharField(max_length=256)
    no_of_packages = models.IntegerField(max_length=20, null=True)
    weight = models.IntegerField(max_length=20, null=True)
    value = models.IntegerField(max_length=20, null=True)

    def __unicode__(self):
        return self.description


class Translation(models.Model):
    string = models.TextField(unique=True)
    string_trans = models.TextField(blank=True, null=True)
    looked_at = models.NullBooleanField(default=False)
#    user = models.ForeignKey(User)

    def __unicode__(self):
        return "%s: %s" % (self.string, self.string_trans,)

    class Meta:
        ordering = ('id',)

databrowse.site.register(Ship, Good, Manifest)
